/*
 * Copyright (C) 2012 Soomla Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#import "VirtualItemStorage.h"
#import "VirtualItem.h"
#import "StorageManager.h"
#import "StoreEncryptor.h"
#import "KeyValDatabase.h"
#import "StoreUtils.h"

@implementation VirtualItemStorage

- (int)balanceForItem:(VirtualItem*)item{
    LogDebug(tag, ([NSString stringWithFormat:@"trying to fetch balance for virtual item with itemId: %@", item.itemId]));
    
    NSString* key = [StoreEncryptor encryptString:[self keyBalance:item.itemId]];
    NSString* val = [[[StorageManager getInstance] kvDatabase] getValForKey:key];
    
    if (!val || [val length]==0){
        return 0;
    }
    
    NSNumber* balance = [StoreEncryptor decryptToNumber:val];
    
    LogDebug(tag, ([NSString stringWithFormat:@"the balance for %@ is: %d", item.itemId, [balance intValue]]));
    
    return [balance intValue];
}

- (int)addAmount:(int)amount toItem:(VirtualItem*)item{
    return [self addAmount:amount toItem:item withEvent:YES];
}
- (int)addAmount:(int)amount toItem:(VirtualItem*)item withEvent:(BOOL)notify {
    LogDebug(tag, ([NSString stringWithFormat:@"adding %d %@", amount, item.name]));
    
    NSString* key = [StoreEncryptor encryptString:[self keyBalance:item.itemId]];
    int balance = [self balanceForItem:item] + amount;
    [[[StorageManager getInstance] kvDatabase] setVal:[StoreEncryptor encryptNumber:[NSNumber numberWithInt:balance]] forKey:key];

    if (notify) {
        [self postBalanceChangeToItem:item withBalance:balance andAmountAdded:amount];
    }
    
    return balance;
}

- (int)removeAmount:(int)amount fromItem:(VirtualItem*)item{
    return [self removeAmount:amount fromItem:item withEvent:YES];
}

- (int)removeAmount:(int)amount fromItem:(VirtualItem*)item withEvent:(BOOL)notify {
    LogDebug(tag, ([NSString stringWithFormat:@"removing %d %@", amount, item.name]));
    
    NSString* key = [StoreEncryptor encryptString:[self keyBalance:item.itemId]];
    int balance = [self balanceForItem:item] - amount;
	if (balance < 0) {
	    balance = 0;
	    amount = 0;
	}
    [[[StorageManager getInstance] kvDatabase] setVal:[StoreEncryptor encryptNumber:[NSNumber numberWithInt:balance]] forKey:key];
    
    if (notify) {
        [self postBalanceChangeToItem:item withBalance:balance andAmountAdded:(-1*amount)];
    }
    
    return balance;
}

- (int)setBalance:(int)balance toItem:(VirtualItem*)item {
    return [self setBalance:balance toItem:item withEvent:YES];
}
- (int)setBalance:(int)balance toItem:(VirtualItem*)item withEvent:(BOOL)notify {
    LogDebug(tag, ([NSString stringWithFormat:@"setting balance %d to %@", balance, item.name]));
    
    int oldBalance = [self balanceForItem:item];
    if (oldBalance == balance) {
        return balance;
    }
    
    NSString* key = [StoreEncryptor encryptString:[self keyBalance:item.itemId]];
    [[[StorageManager getInstance] kvDatabase] setVal:[StoreEncryptor encryptNumber:[NSNumber numberWithInt:balance]] forKey:key];
    
    if (notify) {
        [self postBalanceChangeToItem:item withBalance:balance andAmountAdded:0];
    }
    
    return balance;
}

- (NSString*)keyBalance:(NSString*)itemId {
    @throw [NSException exceptionWithName:NSInternalInconsistencyException
                                   reason:[NSString stringWithFormat:@"You must override %@ in a subclass",
                                           NSStringFromSelector(_cmd)]
                                 userInfo:nil];
}

- (void)postBalanceChangeToItem:(VirtualItem*)item withBalance:(int)balance andAmountAdded:(int)amountAdded {
    @throw [NSException exceptionWithName:NSInternalInconsistencyException
                                   reason:[NSString stringWithFormat:@"You must override %@ in a subclass",
                                           NSStringFromSelector(_cmd)]
                                 userInfo:nil];
}

@end
